@@ -1,3 +1,10 @@
+# Python-bioformats is distributed under the GNU General Public
+# License, but this file is licensed under the more permissive BSD
+# license.  See the accompanying file LICENSE for details.
+# 
+# Copyright (c) 2009-2014 Broad Institute
+# All rights reserved.
+
 '''formatreader.py - mechanism to wrap a bioformats ReaderWrapper and ImageReader
 
 Example:
@@ -13,23 +20,14 @@ Example:
     my_red_image, my_green_image, my_blue_image = \
         [cs.open_bytes(cs.getIndex(0,i,0)) for i in range(3)]
 
-CellProfiler is distributed under the GNU General Public License,
-but this file is licensed under the more permissive BSD license.
-See the accompanying file LICENSE for details.
-
-Copyright (c) 2003-2009 Massachusetts Institute of Technology
-Copyright (c) 2009-2014 Broad Institute
-All rights reserved.
-
-Please see the AUTHORS file for credits.
-
-Website: http://www.cellprofiler.org'
 '''
     
 __version__ = "$Revision$"
 
 import logging
 logger = logging.getLogger(__name__)
+import errno
+import exceptions
 import numpy as np
 import os
 import sys
@@ -37,6 +35,7 @@ import urllib
 import urllib2
 import shutil
 import tempfile
+import traceback
 
 import javabridge as jutil
 import bioformats
@@ -344,6 +343,32 @@ def make_reader_wrapper_class(class_name):
                                   'Set the name of the data file')
     return ReaderWrapper
 
+__has_omero_jars = None
+def has_omero_packages():
+    '''Return True if we can find the packages needed for OMERO
+    
+    In order to run OMERO, you'll need the OMERO client and ICE
+    on your class path (not supplied with python-bioformats and
+    specific to your server's version)
+    '''
+    global __has_omero_jars
+    if __has_omero_jars is None:
+        class_loader = jutil.static_call(
+            "java/lang/ClassLoader", "getSystemClassLoader",
+            "()Ljava/lang/ClassLoader;")
+        for klass in ("Glacier2.PermissionDeniedException", 
+                      "loci.ome.io.OmeroReader", "omero.client"):
+            try:
+                jutil.call(
+                    class_loader, "loadClass", 
+                    "(Ljava/lang/String;)Ljava/lang/Class;", klass)
+            except:
+                __has_omero_jars = False
+                break
+        else:
+            __has_omero_jars = True
+    return __has_omero_jars
+
 __omero_server = None
 __omero_username = None
 __omero_session_id = None
@@ -546,9 +571,6 @@ class ImageReader(object):
                             omero_logout()
                             omero_login()
                         else:
-                            import errno
-                            import exceptions
-                            import traceback
                             logger.warn(e.message)
                             for line in traceback.format_exc().split("\n"):
                                 logger.warn(line)
@@ -585,6 +607,12 @@ class ImageReader(object):
                 self.path = self.path.replace("/", os.path.sep)
             filename = os.path.split(path)[1]
 
+        if not os.path.isfile(self.path):
+            raise exceptions.IOError(
+                errno.ENOENT, 
+                "The file, \"%s\", does not exist." % path,
+                path)
+            
         self.stream = jutil.make_instance('loci/common/RandomAccessInputStream',
                                           '(Ljava/lang/String;)V', 
                                           self.path)
@@ -663,19 +691,16 @@ class ImageReader(object):
         mdoptions = metadatatools.get_metadata_options(metadatatools.ALL)
         self.rdr.setMetadataOptions(mdoptions)
         self.rdr.setGroupFiles(False)
-        metadata = metadatatools.createOMEXMLMetadata()
-        self.rdr.setMetadataStore(metadata)
+        self.metadata = metadatatools.createOMEXMLMetadata()
+        self.rdr.setMetadataStore(self.metadata)
         try:
             self.rdr.setId(self.path)
         except jutil.JavaException, e:
-            import errno
-            import exceptions
-            import traceback
             logger.warn(e.message)
             for line in traceback.format_exc().split("\n"):
                 logger.warn(line)
             je = e.throwable
-            if jutil.is_instance_of(
+            if has_omero_packages() and jutil.is_instance_of(
                 je, "Glacier2/PermissionDeniedException"):
                 # Handle at a higher level
                 raise
@@ -722,7 +747,7 @@ class ImageReader(object):
         pixel_type = self.rdr.getPixelType()
         little_endian = self.rdr.isLittleEndian()
         if pixel_type == FormatTools.INT8:
-            dtype = np.char
+            dtype = np.int8
             scale = 255
         elif pixel_type == FormatTools.UINT8:
             dtype = np.uint8
@@ -750,7 +775,7 @@ class ImageReader(object):
             try:
                 scale = jutil.call(max_sample_value, 'intValue', '()I')
             except:
-                bioformats.logger.warning("WARNING: failed to get MaxSampleValue for image. Intensities may be improperly scaled.")
+                logger.warning("WARNING: failed to get MaxSampleValue for image. Intensities may be improperly scaled.")
         if index is not None:
             image = np.frombuffer(self.rdr.openBytes(index), dtype)
             if len(image) / height / width in (3,4):
@@ -792,7 +817,7 @@ class ImageReader(object):
             image = np.dstack(images)
             image.shape = (height, width, self.rdr.getSizeC())
             if not channel_names is None:
-                metadata = metadatatools.MetadataRetrieve(metadata)
+                metadata = metadatatools.MetadataRetrieve(self.metadata)
                 for i in range(self.rdr.getSizeC()):
                     index = self.rdr.getIndex(z, 0, t)
                     channel_name = metadata.getChannelName(index, i)
